/* Reflow Oven Controller
 *
 * Copyright (C) 2020  Mario Hüttel <mario.huettel@gmx.net>
 *
 * This file is part of the Reflow Oven Controller Project.
 *
 * The reflow oven controller is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * GDSII-Converter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the reflow oven controller project.
 * If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __CRC_UNIT_H__
#define __CRC_UNIT_H__

#include <stdint.h>

void crc_unit_init(void);

void crc_unit_deinit(void);

void crc_unit_reset(void);

uint32_t crc_unit_get_crc(void);

void crc_unit_input(uint32_t data);

void crc_unit_input_array(const uint32_t *data, uint32_t len);

#endif /* __CRC_UNIT_H__ */
